var ISML = require('dw/template/ISML');
var OrderMgr = require('dw/order/OrderMgr');
var Transaction = require('dw/system/Transaction');
var paymentService = require('*/cartridge/scripts/payment/paymentService');
var SaferpayTransactionService = require('*/cartridge/scripts/services/saferpayTransactionService');
var OrderModel = require('*/cartridge/models/order');
var Logger = require('*/cartridge/scripts/utils/logger');
var orderHelper = require('*/cartridge/scripts/order/orderHelper');
var Order = require('dw/order/Order');

var renderTemplate = function (templateName, viewParams) {
    try {
        ISML.renderTemplate(templateName, viewParams);
    } catch (e) {
        Logger.error('Error while rendering template ' + templateName);
        throw e;
    }
};

// TODO: Check if order is authenticated through service
var isCancelAllowed = function (orderNo) {
    const order = OrderMgr.getOrder(orderNo);
    if (!order) return false;
    const orderStatus = order.status.value;
    return orderStatus === Order.ORDER_STATUS_OPEN ||
        orderStatus === Order.ORDER_STATUS_NEW ||
        orderStatus === Order.ORDER_STATUS_COMPLETED;
};

exports.Start = function () {
    const orderNo = request.httpParameterMap.get('order_no').stringValue;

    if (!isCancelAllowed(orderNo)) {
        renderTemplate('order/payment/cancel/order_payment_cancel_not_available.isml');
    } else {
        var order = OrderMgr.getOrder(orderNo);

        renderTemplate('order/payment/cancel/order_payment_cancel.isml', {
            order: new OrderModel(order, {
                containerView: 'order'
            })
        });
    }
};

exports.Cancel = function () {
    const orderNo = request.httpParameterMap.get('orderId').stringValue;
    const order = OrderMgr.getOrder(orderNo);

    const viewParams = {
        success: true,
        orderId: orderNo
    };

    try {
        var cancelTransactionResult;

        Transaction.wrap(function () {
            var saferpayTransaction = paymentService.getPaymentTransaction(order);
            if (saferpayTransaction && saferpayTransaction.isAuthorised()) {
                cancelTransactionResult = SaferpayTransactionService.cancelTransaction({
                    transactionId: saferpayTransaction.id
                });
                orderHelper.addItemToOrderHistory(order, 'SAFERPAY :: CancelTransactionResult: ' + cancelTransactionResult.raw);
                orderHelper.cancelOrder(order, 'PAYMENT :: ERROR :: order manually cancelled');
            } else {
                orderHelper.cancelOrder(order, 'PAYMENT :: ERROR :: order manually cancelled');
            }
        });
    } catch (e) {
        Logger.error('PAYMENT :: ERROR :: Error while processing payment for order ' + orderNo + '. ' + e.message);
        viewParams.success = false;
        viewParams.errorMessage = e.message;
    }

    renderTemplate('order/payment/cancel/order_payment_cancel_confirmation.isml', viewParams);
};

exports.Start.public = true;
exports.Cancel.public = true;
