const { stubs } = testHelpers;
const Ajv = require('ajv');
const ajv = new Ajv({ allErrors: true });
const validate = ajv.compile(require('../saferpayServiceSchemas').deleteAlias);

const proxyquire = require('proxyquire').noCallThru().noPreserveCache();
const deleteAlias = proxyquire('../../../../../../../cartridges/int_saferpay_sfra/cartridge/scripts/services/saferpay/alias/deleteAlias', {
    '*/cartridge/scripts/utils/logger': stubs.loggerMock
});

describe('saferpay/deleteAlias', () => {
    before(function () { stubs.init(); });
    afterEach(function () { stubs.reset(); });
    after(function () { stubs.restore(); });

    context('#payloadBuilder', () => {
        it('builds a correct payload', () => {
            const payload = deleteAlias.payloadBuilder({ aliasId: faker.random.alphaNumeric(40) });
            validate(payload);
            expect(validate(payload)).to.be.eql(true, JSON.stringify(validate.errors));
        });
    });

    context('#responseMapper', () => {
        it('returns a parsed response object', () => {
            const result = {};
            const response = deleteAlias.responseMapper(result);
            expect(response.raw).to.eql(JSON.stringify(result));
        });

        it('handles result without expected properties', () => {
            let response = deleteAlias.responseMapper({});
            expect(response).to.eql({ raw: JSON.stringify({}) });
        });

        it('handles a null or undefined result', () => {
            let response = deleteAlias.responseMapper(null);
            expect(response).to.eql({ raw: null });

            response = deleteAlias.responseMapper();
            expect(response).to.eql({ raw: null });
        });

        it('handles a string result', () => {
            const response = deleteAlias.responseMapper('string');
            expect(response).to.eql({ raw: 'string' });
        });
    });
});
